@@ -3,7 +3,9 @@ import logging
 import asyncio
 import typing
 from ccxt.base import errors as ccxt_errors
-import ccxt.async_support as ccxt
+import ccxt.pro as ccxt
+
+# import ccxt.async_support as ccxt
 from decimal import Decimal
 from datetime import datetime, timedelta, timezone
 from tortoise import connections
@@ -44,7 +46,7 @@ class ZCCXT:
         None  # unable to set ccxt.Exchange type because all methods in that base class raise NotImplemented exception
     )
 
-    def __init__(self, api_key: str, api_secret: str, exchange: ExchangeEnum, is_demo=False, account_address=None):
+    def __init__(self, api_key: str, api_secret: str, exchange: ExchangeEnum, is_demo=False, identifier=None):
         if exchange.value not in [e.value for e in ExchangeEnum]:
             raise ECInvalidValue(f"Invalid exchange provided: {exchange}")
 
@@ -52,11 +54,13 @@ class ZCCXT:
         self.api_secret = api_secret
         self.exchange = exchange
         self.is_demo = is_demo
-        self.account_address = account_address  # used for Hyperliquid
+        self.identifier = identifier
 
     @classmethod
-    async def create(cls, api_key: str, api_secret: str, exchange: ExchangeEnum, is_demo=False, account_address=None):
-        self = cls(api_key, api_secret, exchange, is_demo, account_address)
+    async def create(
+        cls, api_key: str, api_secret: str, exchange: ExchangeEnum, is_demo=False, identifier=None
+    ):
+        self = cls(api_key, api_secret, exchange, is_demo, identifier)
 
         params = {
             "apiKey": self.api_key,
@@ -70,12 +74,14 @@ class ZCCXT:
             params.pop("secret", None)
             params.update(
                 {
-                    "walletAddress": self.account_address,
+                    "walletAddress": self.identifier,
                     "enableRateLimit": True,
                     # "rateLimit": 1200,
                     "privateKey": self.api_secret,
                 }
             )
+        elif exchange == ExchangeEnum.WOO:
+            params.update({"uid": self.identifier})
 
         self.client = getattr(ccxt, self.exchange.lower())(params)
 
@@ -84,12 +90,17 @@ class ZCCXT:
             "SELECT * FROM ccxt_market_cache WHERE exchange = $1",
             (self.exchange.value,),
         )
-        key = res[0]
-        if key["updated_at"] <= datetime.now(timezone.utc) - timedelta(hours=3):
-            markets = await self.client.load_markets(reload=True)
-            await self.cache_data(markets)
+
+        if len(res) > 0:
+            key = res[0]
+            if key["updated_at"] <= datetime.now(timezone.utc) - timedelta(hours=3):
+                markets = await self.client.load_markets(reload=True)
+                await self.cache_data(markets)
+            else:
+                self.client.set_markets(json.loads(key["data"]))
         else:
-            self.client.set_markets(json.loads(key["data"]))
+            markets = self.client.load_markets(reload=True)
+            await self.cache_data(markets)
 
         if self.is_demo:
             self.client.enableDemoTrading(True)
@@ -113,11 +124,7 @@ class ZCCXT:
         """
         Fetch balances from the exchange
         """
-        params = {}
-        if self.exchange == ExchangeEnum.HYPERLIQUID:
-            params = {"user": self.account_address}
-
-        tasks = [self.client.fetch_balance(params=params)]
+        tasks = [self.client.fetch_balance()]
 
         if self.exchange == ExchangeEnum.BYBIT:
             tasks.append(self.client.privateGetV5AccountInfo())
@@ -127,8 +134,7 @@ class ZCCXT:
             tasks.append(self.get_positions())
         elif self.exchange == ExchangeEnum.HYPERLIQUID:
             # for HL we need to fetch balance twice, once for futures and another time for spot
-            params.update({"type": "spot"})
-            tasks.append(self.client.fetch_balance(params=params))
+            tasks.append(self.client.fetch_balance(params={"type": "spot"}))
             tasks.append(self.get_positions())
 
         responses = await asyncio.gather(*tasks)
@@ -173,7 +179,7 @@ class ZCCXT:
 
             try:
                 resp = await self.get_order(order_id, market)
-            except ccxt.OrderNotFound:
+            except ccxt_errors.OrderNotFound:
                 log.debug("Order %s not found, retries left: %d", order_id, max_retries)
                 max_retries -= 1
                 await asyncio.sleep(0.5)
@@ -201,6 +207,33 @@ class ZCCXT:
         """
         return await self.client.fetch_order_book(symbol)
 
+    async def _get_order(
+        self,
+        order_id: str,
+        market: typing.Optional[MarketTypeEnum] = MarketTypeEnum.SPOT,
+        order_type: typing.Optional[typing.Union["open", "closed"]] = None,
+    ):
+        try:
+            if self.exchange == ExchangeEnum.BYBIT:
+                market_value = market.value
+                if market_value == "future" or market_value == "swap":
+                    market_value = "linear"
+
+                if order_type == "open":
+                    return await self.client.fetch_open_order(order_id, params={"category": market_value})
+                return await self.client.fetch_closed_order(order_id, params={"category": market_value})
+
+            return await self.client.fetch_order(order_id)
+        except ccxt_errors.OrderNotFound:
+            return None
+        except ccxt_errors.BadRequest as e:
+            assert True
+            if "order can not be found" in e.args[0]:
+                return None
+            raise e
+        except Exception as e:
+            raise e
+
     async def get_order(
         self,
         order_id: str,
@@ -211,17 +244,13 @@ class ZCCXT:
         Fetch an order from the exchange
         """
         if self.exchange == ExchangeEnum.BYBIT:
-            market_value = market.value
-            if market_value == "future" or market_value == "swap":
-                market_value = "linear"
-
-            if order_type == "open":
-                return await self.client.fetch_open_order(order_id, params={"category": market_value})
-            return await self.client.fetch_closed_order(order_id, params={"category": market_value})
+            return await self._get_order(order_id, market, order_type)
         elif self.exchange == ExchangeEnum.WOO:
-            return await self.client.fetch_order(order_id)
+            return await self._get_order(order_id)
         elif self.exchange == ExchangeEnum.HYPERLIQUID:
-            order = await self.client.fetch_order(order_id)
+            order = await self._get_order(order_id)
+            if not order or not order["id"]:
+                return None
 
             # set default values in case we return early
             order.update(
@@ -563,7 +592,7 @@ class ZCCXT:
             )
 
         elif self.exchange == ExchangeEnum.HYPERLIQUID:
-            params = {"type": "userFunding", "user": self.account_address}
+            params = {"type": "userFunding", "user": self.identifier}
             if symbol:
                 params.update({"symbol": symbol})
             if from_dt:
@@ -769,7 +798,6 @@ class ZCCXT:
             response_spot = response_data["balance_spot"]
             response = response_data["balance"]
             positions = response_data["positions"]
-            max_leverage = 3  # we play it safe and hardcode a 3 to avoid getting rekt ... Note that user might use 1 in the  HL dashboard
             total_balance = Decimal(response["total"]["USDC"]).quantize(Decimal("0.00000001"))
             margin_balance = Decimal(response["used"]["USDC"]).quantize(Decimal("0.00000001"))
             free_margin = Decimal(response["free"]["USDC"]).quantize(Decimal("0.00000001"))
@@ -779,7 +807,7 @@ class ZCCXT:
             total_notional = Decimal(sum(Decimal(position["notional"]) for position in positions)).quantize(
                 Decimal("0.00000001")
             )
-            maintenance_margin = total_notional / (2 * max_leverage)
+            maintenance_margin = Decimal(response["info"]["crossMaintenanceMarginUsed"])
             maintenance_margin_ratio = maintenance_margin / total_balance if maintenance_margin else Decimal(0)
             margin_ratio = (margin_balance / total_notional if total_notional else Decimal(0)).quantize(
                 Decimal("0.00000001")
